;
; Kaminsimulation fr Board AT13 
; GH 18.12.2009, AVR-Studio 4.15
; 
; Funktion: 
; 	Ausgabe von 5 Zeitfunktionen a 256 Schritten
; 	als Pulsweitenmodulation fr fnf LEDs an PB0-4
;
; Prozessor: ATMEL ATtiny13 mit 5V
;
.include "tn13def.inc"	; Prozessordaten
.device attiny13		; Assembler directive
; 
; Pinout 8-PDIP/SOIC: 	
; 				 --------
; 	/RESET,	PB5	|1 	`	8| VCC
; 			PB3	|2		7| PB2
; 			PB4	|3		6| PB1
; 			GND	|4		5| PB0
; 	 			 --------
; Ports:
; 
; 	Port	Register	Farbe	R in Ohm
; 	PB0:	led1		gelb1	150
; 	PB1:	led2		rot1	150
; 	PB2:	led3		weiss	150
; 	PB3:	led4		rot2	150
; 	PB4:	led5		gelb2	150
;
; LEDs gegen VCC geschaltet
; Nachschaltung eines PowerMOS invertiert, d.h. 1 ist hell
; u.U. knnen LEDs direkt getrieben werden ohne PowerMOS
; Belastung pro Pin dann unter 20 mA halten!

;############### Namen ###################

.equ outp = PORTB		; default-Ausgabeport
.equ dirp = DDRB		; Datenrichtung Ausgabeport
.equ pinp = PINB		; Eingabeport

; Pingruppen Zuordnung (Form: PB7...PB0)
.equ hell 	= 0b0001_1111	; PB0-4 alle hell
.equ dunkl  = 0b0000_0000	; PB0-4 alle dunkel
.equ gelb 	= 0b0001_0001	; gelbe hell
.equ weiss 	= 0b0000_0100	; weiss hell
.equ rot 	= 0b0000_1010	; rote  hell

; Pins einzeln ansteuern (1 ist hell)
.equ k0 	= 0b0000_0001	; gb1 hell
.equ k1 	= 0b0000_0010	; rt1 hell
.equ k2 	= 0b0000_0100	; ws  hell
.equ k3 	= 0b0000_1000	; rt2 hell
.equ k4 	= 0b0001_0000	; gb2 hell

;.equ delay = 0x01		; PWM Taktzeit (grer null)
.equ dauer = 0x60		; Lauflicht-Verzgerung pro Stufe


; Register:
.def 	a1 		= r16	; Arbeit
.def	a2		= r17	; Arbeit
.def	a3		= r18	; Arbeit
.def	delay	= r19	; PWM Taktzeit
.def 	zahl	= r20	; Zufallszahl
.def 	pwm		= r21	; PWM-Zhler

.def 	summe	= r22	; temp-Register Zufallszahlen
.def 	null	= r23	; temp-Register Zufallszahlen
.def 	zykl	= r24	; Zyklenzhler
.def 	leds	= r25	; LED Outport (1 hell, 0 dunkel)

; XL r26 frei

; Helligkeits-Register: XH...ZH belegt:
; XH (Register r27)				; PWM-Dauer 
; YL, YH (Register r28, r29) 	; PWM-Dauer 
; ZL, ZH (Register r30, r31) 	; PWM-Dauer 

;############### Adressen ################


ldi r16,low(RAMEND)		; Set Stack Pointer to top of RAM
out SPL,r16 			; an den Stackpointer (nur low)

.org 0x0010				; Sprungadressen der Interrupts auslassen
.cseg					; Codesegment folgt

rjmp Init				; dort gehts los


; **********************************
; 		Unterprogramme
; **********************************


zufall:	; Zufallszahl fr Helligkeit erzeugen
		; rckgekoppeltes Schieberegister (8 Bit random)
		; inp: zahl; outp: zahl
	mov a1,zahl
	clr summe
	clr null
	lsr a1

	adc summe,null
	lsr a1
	lsr a1
	lsr a1
	lsr a1
	adc summe,null
	lsr a1
	adc summe,null
	lsr a1
	adc summe,null

	lsr zahl
	sbrc summe, 0
	sbr zahl,128
ret


warte:	; Warten, Rckwrts zhlen um max. 2^16-1
		; max a3 = 255, min a3 = 0
		; inp: a3, delay; outp: Verzgerung

		inc a3		; damit a3 = 0...255
		wart1:
		mov a2,delay	; Grunddelay erzeugen
			wart2:
			dec		a2
			cpi 	a2,0
			brne	wart2	; branch if not equal
		dec		a3
		cpi 	a3,0
		brne	wart1	; branch if not equal
ret						; Rcksprung



horu:	; hoch/runter, Sgezahn in Dreieck wandeln
		; Input 0...127...255 in Output 0...255...0 wandeln
		; inp = outp = a1
	
	sbrc a1,7		; berspringe, wenn MSB = 0, d.h. Zahl kleiner 128 (unsigned)
	com a1			; alle Kleinen negieren
					; die Groen so lassen

	hoend:			; alle Werte sind nun zwischen 128 und 255
	subi a1,0x7f	; 127 abziehen um auf 0...127 zu kommen
	lsl a1			; mal zwei um auf 0...255 zu kommen
ret



; **********************************
; 			Start, Init
; **********************************

Init:

; Port
	ldi	a1,hell		; Portinit, alle hell
	out dirp,a1		; Datenrichtung Ausgabe
	ldi leds, 0x00	; Led-Grundwert 00: dunkel

; Zufall
	ldi zahl,1  	; init Zufallszahl
	ldi zykl,0  	; init Zyklus
	ldi pwm,0		; lange Schleife

; Timerregister	Init Mittelstellung 0x80 = 128 dez
	ldi XH, 0x80	; gb1
	ldi YL, 0x80	; rt1
	ldi YH, 0x80	; ws
	ldi ZL, 0x80	; rt2
	ldi ZH, 0x80	; gb2

; Funktion warte initialisieren
	ldi a3,0xff		; lange warten, Init fr warte
	ldi delay,0xff	; lange warten, Init fr warte


; LEDs einzeln ein- und ausschalten:
	ldi a1,k0		; k0 an Ausgang senden, warten
	out outp,a1
	rcall warte

	ldi a1,k1
	out outp,a1
	rcall warte

	ldi a1,k2
	out outp,a1
	rcall warte

	ldi a1,k3
	out outp,a1
	rcall warte

	ldi a1,k4
	out outp,a1
	rcall warte

; Gruppen testen:
	ldi a1,dunkl
	out outp,a1
	rcall warte

	ldi a1,rot
	out outp,a1
	rcall warte

	ldi a1,gelb
	out outp,a1
	rcall warte

	ldi a1,weiss
	out outp,a1
	rcall warte
	
	ldi a1,hell
	out outp,a1
	rcall warte

; Init Ende:
	ldi delay,0x01	; PWM-Takt initialisieren


; **********************************
; 		Main
; **********************************

main:				; ab jetzt mit Pulsweitenmodulation (PWM)
inc zykl			; Zyklenzhler erhhen
cpi zykl, dauer		; nur bei Gleichheit neu einlesen
brne rhein			; wenn zykl ungleich dauer: nichts verndern
ldi zykl,0			; bei zykl=0 neue Zufallszahl erzeugen

; Zufallszahl zahl erzeugen (inp = outp = zahl)
	rcall zufall		; Zufallszahl erzeugen auf zahl
	mov a1,zahl			; Zufallszahl auf Reg. a1

; Zufall und Statik dezent mischen
	clc		; clear carry flag, falls lsr
	lsr a1	; Division durch 2, Zufall nicht zu stark whlen
	lsr a1	; nochmal
	;lsr a1	; nochmal
	;lsr a1	; nochmal
	add a1,leds		; Wert leds addieren

; Zufallszahl in hoch-runter konvertieren
	rcall horu	; auf a1

; Werte ber die Zeitregister schieben
	mov ZH,ZL	; weiterschieben
	com ZL
	mov ZL,YH	; -"-
	com YH
	mov YH,YL	; -"-
	com YL
	mov YL,XH	; -"-
	com XH
	mov XH,a1	; nchsten Wert holen


rhein:	
	; PWM erzeugen, 256 Takte
	ldi pwm,0		; init PWM-Zhler
	ldi a1,0		; LED-Portout Register einstellen
	out outp,a1		; alle LEDs leuchten jetzt mit a1

pwmloop:
	; PWM: for pwm = 1 to 255 (fr jede Zeitstufe)
	; LED-Bits nach Zeitdauer in YL...ZH umschalten

	cp YL,pwm	; Zeit fr k0 abgelaufen?
	brsh w1		; same or higher? Dann drber. !bug brlo: signed, test auf S!
	sbi outp,0	; Bit k0 setzen
	w1:

	cp YH,pwm	; k1
	brsh w2
	sbi outp,1	; Bit k1 setzen
	w2:

	cp ZL,pwm	; k2
	brsh w3
	sbi outp,2	; Bit k2 setzen
	w3:

	cp ZH,pwm	; k3
	brsh w4
	sbi outp,3	; Bit k3 setzen
	w4:

	cp XH,pwm	; k4
	brsh w5
	sbi outp,4	; Bit k4 setzen
	w5:


	; warten
	mov a3, delay	; einen PWM-Schritt lang warten
	rcall warte

	inc pwm			; nchster PWM-Zeitschritt
	cpi pwm, 0		; PWM abgelaufen, ganz von vorn
	breq main

rjmp pwmloop		; in Schleife bleiben

rjmp main


;**************************************************************





